/*
This program is free software: you can redistribute it and/or modify
 it under the terms of the Attribution 3.0 Unported as published by
 Creative Commons.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 
 You should have received a copy of the Attribution 3.0 Unported or visit
 http://creativecommons.org/licenses/by/3.0/legalcode
 
 Original Author: Jeffrey Nygaard
 Company: Northern Lights Electronic Design, LLC
 Date: March 6, 2014
 Author Contact: JNygaard@NLEDshop.com
 Support Contact: Support@NLEDshop.com
 Written in Processing v2.0.3

 www.NLEDshop.com
 www.NorthernLightsElectronicDesign.com
 http://www.processing.org/
  
 This software is offered for free and is open-source. You may use it for your commercial projects as described in the license. You may modify
 it to suit you, and share your code if you wish, as long as you give attribution and release it under the same license. It is encouraged to submit 
 improvements and bug fixes so they can be added to the main release version.
 
 /*===========================================================*\
 Created for use with compatible Northern Lights Electronic Design LED Pixel Controllers
 NLED Pixel Controller Mini - www.NLEDshop.com/pixelmini
 
 Used G4P and GUI builder to create the GUI
 http://www.lagers.org.uk/g4p/
 http://www.lagers.org.uk/g4p/ref/index.html
 */

//Should probably make SPBRG values referecned in the firmware, to make it more dynamic
//  255uS to 32mS

import g4p_controls.*;
import processing.serial.*;

final color cBackgroundColor = color(200); //background of the software

int inByte = -1;    // Incoming serial data

Serial myPort;      // The serial port
DropDown COMPortDD;

void setup() 
{
  size(400, 400);
  textFont(loadFont("SansSerif.bold-48.vlw"), 48); //font saved in datat folder
  
  createGUI();
  customGUI();
  
  textfield3.setText("0"); //makes sure it won't have space, "" still places a space
  
  println(Serial.list());

  //custom drop down for COM Ports
  COMPortDD = new DropDown(Serial.list(), Serial.list().length, 0, 290, 60, 100, 25, color(220), color(220, 220, 255), color(0), false);
} //end setup()

void draw() {
  background(cBackgroundColor);
  COMPortDD.display();
}

void serialEvent(Serial myPort) {
  inByte = myPort.read();
}

public void customGUI() {
}

//========================== DROP DOWN OBJECT FOR COM PORT SELECTION ============
class DropDown
{
  //use one over() for detecting the button click and if it is open, then use a different over() with the full tab page size
  String[] Labels;
  int numStrs;
  int SelStr;
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int bWidth;
  int bHeight;
  color ColBG;
  color ColHL;
  color ColOut;  
  boolean Selected;
  //  boolean Opened;

  DropDown(String[] iLabels, int inumStrs, int iSelStr, int ixpos, int iypos, int ibWidth, int ibHeight, color iColBG, color iColHL, color iColOut, boolean iSelected)//, boolean iOpened)
  {
    Labels = iLabels; 
    numStrs = inumStrs;
    SelStr = iSelStr;
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = ibHeight;
    ColBG = iColBG;
    ColHL = iColHL;
    ColOut = iColOut;
    Selected = iSelected;
    //Opened = iOpened;
  }

  void displayNormal()
  {
    strokeWeight(1);
    stroke(ColOut);

    if (Selected==true) fill(ColHL);
    else fill(ColBG);

    rect(xpos, ypos, bWidth, bHeight);  
    //adjust stroke to on higlight and invert text color
    fill(ColOut);
    textSize((bHeight/1.75));
    text(Labels[SelStr], xpos+(bWidth/20), ypos+((bHeight-(textAscent()+textDescent()))/1.5), bWidth, bHeight);

    fill(255);
    int tempSize = (bWidth/4);
    if (tempSize >  (bHeight/4)) tempSize = (bHeight);
    triangle(xpos+(bWidth-tempSize), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/2), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/4), ypos+(bHeight/4)+(bHeight/2));
    noStroke();
  }

  void display()
  {

    if (Selected == false)
    {
      displayNormal();
    }
    else
    {
      fill(ColBG);
      displayNormal();
      strokeWeight(1);
      stroke(ColOut);  

      for (int i = 0; i != numStrs; i++)
      {
        fill(ColBG);
        rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);
        fill(ColOut);
        textSize((bHeight/1.75));
        text(Labels[i], 5+xpos+((bWidth - textWidth(Labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5)+(bHeight*(i+1)), bWidth, bHeight);
      }   //end for 
      noStroke();
    }
  }

  void displayOpen()
  {
    fill(ColBG);
    display();
    strokeWeight(1);
    stroke(ColOut);  

    for (int i = 0; i != numStrs; i++)
    {
      fill(ColBG);
      rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);
      fill(ColOut);
      textSize((bHeight/1.75));
      text(Labels[i], 5+xpos+((bWidth - textWidth(Labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5)+(bHeight*(i+1)), bWidth, bHeight);
    }   //end for 
    noStroke();
  }

  boolean over() ///over the icon to open
  {
    if ( mouseX >= xpos && mouseX <= xpos+bWidth && mouseY >= ypos && mouseY <= ypos+bHeight) 
    {      
      return true;
    } 
    else {
      return false;
    }
  }

  int overOpen()
  {
    for (int i = 0; i != numStrs; i++)
    {
      if (mouseX >= xpos && mouseX <= xpos+bWidth && mouseY >= ypos+(bHeight*(i+1)) && mouseY <= ypos+bHeight+(bHeight*(i+1)))
      {    
        return (i+1);
      }
    }
    return 0;
  }
}
